# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: shared_types.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12shared_types.proto\x12)ProSuite.Microservices.Definitions.Shared\"\x15\n\x04UUID\x12\r\n\x05value\x18\x01 \x01(\x0c\"I\n\x0b\x45nvelopeMsg\x12\r\n\x05x_min\x18\x01 \x01(\x01\x12\r\n\x05y_min\x18\x02 \x01(\x01\x12\r\n\x05x_max\x18\x03 \x01(\x01\x12\r\n\x05y_max\x18\x04 \x01(\x01\"\xe0\x01\n\x08ShapeMsg\x12\x14\n\nesri_shape\x18\x01 \x01(\x0cH\x00\x12\r\n\x03wkb\x18\x02 \x01(\x0cH\x00\x12J\n\x08\x65nvelope\x18\x03 \x01(\x0b\x32\x36.ProSuite.Microservices.Definitions.Shared.EnvelopeMsgH\x00\x12Y\n\x11spatial_reference\x18\x04 \x01(\x0b\x32>.ProSuite.Microservices.Definitions.Shared.SpatialReferenceMsgB\x08\n\x06\x66ormat\"\xa6\x02\n\x0e\x41ttributeValue\x12\x11\n\x07\x64\x62_null\x18\x01 \x01(\x08H\x00\x12\x19\n\x0fshort_int_value\x18\x02 \x01(\x05H\x00\x12\x18\n\x0elong_int_value\x18\x03 \x01(\x05H\x00\x12\x15\n\x0b\x66loat_value\x18\x04 \x01(\x02H\x00\x12\x16\n\x0c\x64ouble_value\x18\x05 \x01(\x01H\x00\x12\x16\n\x0cstring_value\x18\x06 \x01(\tH\x00\x12\x1f\n\x15\x64\x61te_time_ticks_value\x18\x07 \x01(\x03H\x00\x12\x45\n\nuuid_value\x18\x08 \x01(\x0b\x32/.ProSuite.Microservices.Definitions.Shared.UUIDH\x00\x12\x14\n\nblob_value\x18\t \x01(\x0cH\x00\x42\x07\n\x05value\"\xc6\x01\n\x0cGdbObjectMsg\x12\x11\n\tobject_id\x18\x01 \x01(\x03\x12\x14\n\x0c\x63lass_handle\x18\x02 \x01(\x03\x12I\n\x06values\x18\x03 \x03(\x0b\x32\x39.ProSuite.Microservices.Definitions.Shared.AttributeValue\x12\x42\n\x05Shape\x18\x04 \x01(\x0b\x32\x33.ProSuite.Microservices.Definitions.Shared.ShapeMsg\"7\n\x0cGdbObjRefMsg\x12\x14\n\x0c\x63lass_handle\x18\x01 \x01(\x03\x12\x11\n\tobject_id\x18\x02 \x01(\x03\"\x94\x02\n\x0eObjectClassMsg\x12\x14\n\x0c\x63lass_handle\x18\x01 \x01(\x03\x12\x18\n\x10workspace_handle\x18\x02 \x01(\x03\x12\x0c\n\x04name\x18\x03 \x01(\t\x12\r\n\x05\x61lias\x18\x04 \x01(\t\x12\x15\n\rgeometry_type\x18\x05 \x01(\x05\x12Y\n\x11spatial_reference\x18\x06 \x01(\x0b\x32>.ProSuite.Microservices.Definitions.Shared.SpatialReferenceMsg\x12\x43\n\x06\x66ields\x18\x07 \x03(\x0b\x32\x33.ProSuite.Microservices.Definitions.Shared.FieldMsg\"\xad\x02\n\x0cWorkspaceMsg\x12\x18\n\x10workspace_handle\x18\x01 \x01(\x03\x12\x19\n\x11workspace_db_type\x18\x02 \x01(\x05\x12\x0c\n\x04path\x18\x03 \x01(\t\x12\x14\n\x0cversion_name\x18\x04 \x01(\t\x12\x1c\n\x14\x64\x65\x66\x61ult_version_name\x18\x05 \x01(\t\x12#\n\x1b\x64\x65\x66\x61ult_version_description\x18\x06 \x01(\t\x12&\n\x1e\x64\x65\x66\x61ult_version_creation_ticks\x18\x07 \x01(\x03\x12Y\n\x15\x63onnection_properties\x18\x08 \x03(\x0b\x32:.ProSuite.Microservices.Definitions.Shared.KeyValuePairMsg\"-\n\x0fKeyValuePairMsg\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"\xab\x01\n\x08\x46ieldMsg\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x12\n\nalias_name\x18\x02 \x01(\t\x12\x0c\n\x04type\x18\x03 \x01(\x05\x12\x0e\n\x06length\x18\x04 \x01(\x05\x12\x13\n\x0b\x64omain_name\x18\x05 \x01(\t\x12\r\n\x05scale\x18\x06 \x01(\x05\x12\x11\n\tprecision\x18\x07 \x01(\x05\x12\x13\n\x0bis_nullable\x18\x08 \x01(\x08\x12\x13\n\x0bis_editable\x18\t \x01(\x08\"\x88\x01\n\x13SpatialReferenceMsg\x12$\n\x1aspatial_reference_esri_xml\x18\x01 \x01(\tH\x00\x12\x1f\n\x15spatial_reference_wkt\x18\x02 \x01(\tH\x00\x12 \n\x16spatial_reference_wkid\x18\x03 \x01(\x05H\x00\x42\x08\n\x06\x66ormat\"\xae\x02\n\x0fResultObjectMsg\x12I\n\x06update\x18\x01 \x01(\x0b\x32\x37.ProSuite.Microservices.Definitions.Shared.GdbObjectMsgH\x00\x12N\n\x06insert\x18\x02 \x01(\x0b\x32<.ProSuite.Microservices.Definitions.Shared.InsertedObjectMsgH\x00\x12I\n\x06\x64\x65lete\x18\x03 \x01(\x0b\x32\x37.ProSuite.Microservices.Definitions.Shared.GdbObjRefMsgH\x00\x12\x15\n\rnotifications\x18\x04 \x03(\t\x12\x13\n\x0bhas_warning\x18\x05 \x01(\x08\x42\t\n\x07\x66\x65\x61ture\"\xba\x01\n\x11InsertedObjectMsg\x12P\n\x0finserted_object\x18\x01 \x01(\x0b\x32\x37.ProSuite.Microservices.Definitions.Shared.GdbObjectMsg\x12S\n\x12original_reference\x18\x02 \x01(\x0b\x32\x37.ProSuite.Microservices.Definitions.Shared.GdbObjRefMsgb\x06proto3')



_UUID = DESCRIPTOR.message_types_by_name['UUID']
_ENVELOPEMSG = DESCRIPTOR.message_types_by_name['EnvelopeMsg']
_SHAPEMSG = DESCRIPTOR.message_types_by_name['ShapeMsg']
_ATTRIBUTEVALUE = DESCRIPTOR.message_types_by_name['AttributeValue']
_GDBOBJECTMSG = DESCRIPTOR.message_types_by_name['GdbObjectMsg']
_GDBOBJREFMSG = DESCRIPTOR.message_types_by_name['GdbObjRefMsg']
_OBJECTCLASSMSG = DESCRIPTOR.message_types_by_name['ObjectClassMsg']
_WORKSPACEMSG = DESCRIPTOR.message_types_by_name['WorkspaceMsg']
_KEYVALUEPAIRMSG = DESCRIPTOR.message_types_by_name['KeyValuePairMsg']
_FIELDMSG = DESCRIPTOR.message_types_by_name['FieldMsg']
_SPATIALREFERENCEMSG = DESCRIPTOR.message_types_by_name['SpatialReferenceMsg']
_RESULTOBJECTMSG = DESCRIPTOR.message_types_by_name['ResultObjectMsg']
_INSERTEDOBJECTMSG = DESCRIPTOR.message_types_by_name['InsertedObjectMsg']
UUID = _reflection.GeneratedProtocolMessageType('UUID', (_message.Message,), {
  'DESCRIPTOR' : _UUID,
  '__module__' : 'shared_types_pb2'
  # @@protoc_insertion_point(class_scope:ProSuite.Microservices.Definitions.Shared.UUID)
  })
_sym_db.RegisterMessage(UUID)

EnvelopeMsg = _reflection.GeneratedProtocolMessageType('EnvelopeMsg', (_message.Message,), {
  'DESCRIPTOR' : _ENVELOPEMSG,
  '__module__' : 'shared_types_pb2'
  # @@protoc_insertion_point(class_scope:ProSuite.Microservices.Definitions.Shared.EnvelopeMsg)
  })
_sym_db.RegisterMessage(EnvelopeMsg)

ShapeMsg = _reflection.GeneratedProtocolMessageType('ShapeMsg', (_message.Message,), {
  'DESCRIPTOR' : _SHAPEMSG,
  '__module__' : 'shared_types_pb2'
  # @@protoc_insertion_point(class_scope:ProSuite.Microservices.Definitions.Shared.ShapeMsg)
  })
_sym_db.RegisterMessage(ShapeMsg)

AttributeValue = _reflection.GeneratedProtocolMessageType('AttributeValue', (_message.Message,), {
  'DESCRIPTOR' : _ATTRIBUTEVALUE,
  '__module__' : 'shared_types_pb2'
  # @@protoc_insertion_point(class_scope:ProSuite.Microservices.Definitions.Shared.AttributeValue)
  })
_sym_db.RegisterMessage(AttributeValue)

GdbObjectMsg = _reflection.GeneratedProtocolMessageType('GdbObjectMsg', (_message.Message,), {
  'DESCRIPTOR' : _GDBOBJECTMSG,
  '__module__' : 'shared_types_pb2'
  # @@protoc_insertion_point(class_scope:ProSuite.Microservices.Definitions.Shared.GdbObjectMsg)
  })
_sym_db.RegisterMessage(GdbObjectMsg)

GdbObjRefMsg = _reflection.GeneratedProtocolMessageType('GdbObjRefMsg', (_message.Message,), {
  'DESCRIPTOR' : _GDBOBJREFMSG,
  '__module__' : 'shared_types_pb2'
  # @@protoc_insertion_point(class_scope:ProSuite.Microservices.Definitions.Shared.GdbObjRefMsg)
  })
_sym_db.RegisterMessage(GdbObjRefMsg)

ObjectClassMsg = _reflection.GeneratedProtocolMessageType('ObjectClassMsg', (_message.Message,), {
  'DESCRIPTOR' : _OBJECTCLASSMSG,
  '__module__' : 'shared_types_pb2'
  # @@protoc_insertion_point(class_scope:ProSuite.Microservices.Definitions.Shared.ObjectClassMsg)
  })
_sym_db.RegisterMessage(ObjectClassMsg)

WorkspaceMsg = _reflection.GeneratedProtocolMessageType('WorkspaceMsg', (_message.Message,), {
  'DESCRIPTOR' : _WORKSPACEMSG,
  '__module__' : 'shared_types_pb2'
  # @@protoc_insertion_point(class_scope:ProSuite.Microservices.Definitions.Shared.WorkspaceMsg)
  })
_sym_db.RegisterMessage(WorkspaceMsg)

KeyValuePairMsg = _reflection.GeneratedProtocolMessageType('KeyValuePairMsg', (_message.Message,), {
  'DESCRIPTOR' : _KEYVALUEPAIRMSG,
  '__module__' : 'shared_types_pb2'
  # @@protoc_insertion_point(class_scope:ProSuite.Microservices.Definitions.Shared.KeyValuePairMsg)
  })
_sym_db.RegisterMessage(KeyValuePairMsg)

FieldMsg = _reflection.GeneratedProtocolMessageType('FieldMsg', (_message.Message,), {
  'DESCRIPTOR' : _FIELDMSG,
  '__module__' : 'shared_types_pb2'
  # @@protoc_insertion_point(class_scope:ProSuite.Microservices.Definitions.Shared.FieldMsg)
  })
_sym_db.RegisterMessage(FieldMsg)

SpatialReferenceMsg = _reflection.GeneratedProtocolMessageType('SpatialReferenceMsg', (_message.Message,), {
  'DESCRIPTOR' : _SPATIALREFERENCEMSG,
  '__module__' : 'shared_types_pb2'
  # @@protoc_insertion_point(class_scope:ProSuite.Microservices.Definitions.Shared.SpatialReferenceMsg)
  })
_sym_db.RegisterMessage(SpatialReferenceMsg)

ResultObjectMsg = _reflection.GeneratedProtocolMessageType('ResultObjectMsg', (_message.Message,), {
  'DESCRIPTOR' : _RESULTOBJECTMSG,
  '__module__' : 'shared_types_pb2'
  # @@protoc_insertion_point(class_scope:ProSuite.Microservices.Definitions.Shared.ResultObjectMsg)
  })
_sym_db.RegisterMessage(ResultObjectMsg)

InsertedObjectMsg = _reflection.GeneratedProtocolMessageType('InsertedObjectMsg', (_message.Message,), {
  'DESCRIPTOR' : _INSERTEDOBJECTMSG,
  '__module__' : 'shared_types_pb2'
  # @@protoc_insertion_point(class_scope:ProSuite.Microservices.Definitions.Shared.InsertedObjectMsg)
  })
_sym_db.RegisterMessage(InsertedObjectMsg)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _UUID._serialized_start=65
  _UUID._serialized_end=86
  _ENVELOPEMSG._serialized_start=88
  _ENVELOPEMSG._serialized_end=161
  _SHAPEMSG._serialized_start=164
  _SHAPEMSG._serialized_end=388
  _ATTRIBUTEVALUE._serialized_start=391
  _ATTRIBUTEVALUE._serialized_end=685
  _GDBOBJECTMSG._serialized_start=688
  _GDBOBJECTMSG._serialized_end=886
  _GDBOBJREFMSG._serialized_start=888
  _GDBOBJREFMSG._serialized_end=943
  _OBJECTCLASSMSG._serialized_start=946
  _OBJECTCLASSMSG._serialized_end=1222
  _WORKSPACEMSG._serialized_start=1225
  _WORKSPACEMSG._serialized_end=1526
  _KEYVALUEPAIRMSG._serialized_start=1528
  _KEYVALUEPAIRMSG._serialized_end=1573
  _FIELDMSG._serialized_start=1576
  _FIELDMSG._serialized_end=1747
  _SPATIALREFERENCEMSG._serialized_start=1750
  _SPATIALREFERENCEMSG._serialized_end=1886
  _RESULTOBJECTMSG._serialized_start=1889
  _RESULTOBJECTMSG._serialized_end=2191
  _INSERTEDOBJECTMSG._serialized_start=2194
  _INSERTEDOBJECTMSG._serialized_end=2380
# @@protoc_insertion_point(module_scope)



__all__ = ['Condition']
from prosuite.issue_filter import IssueFilter
from prosuite.parameter import Parameter
from typing import List


class Condition:
    """
    Defines a test condition. Conditions are created with the class :py:mod:`prosuite.generated.quality_conditions` 
    """

    def __init__(self, test_descriptor: str, name: str = ""):

        #:
        self.name = name

        #:
        self.test_descriptor = test_descriptor

        #:
        self.allow_errors: bool = False

        #:
        self.category: str = "Geometry"

        #:
        self.stop_on_error: bool = False

        #:
        self.description: str = ""

        #:
        self.url: str = ""

        #:
        self.parameters: List[Parameter] = []

        #:
        self.issue_filters: List[IssueFilter] = []

        #:
        self.issue_filter_expression: str

    def generate_name(self):
        """
        generates a technical name using the dataset name(s) and the test descriptor 
        """
        first_dataset_parameter = next(
            (p for p in self.parameters if p.is_dataset_parameter), None)
        if first_dataset_parameter:
            ds_param: Parameter = first_dataset_parameter
            if ds_param.contains_list_of_datasets:
                dataset_list: List[str] = [ds.name for ds in ds_param.dataset]
                dataset_names = "_".join(dataset_list)
                self.name = f"{self.test_descriptor} {dataset_names}"
            else:
                self.name = f"{self.test_descriptor} {ds_param.dataset.name}"



# -*- coding: utf-8 -*-
# generated by the protocol buffer compiler.  DO NOT EDIT!
# source: quality_test.proto
"""generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import shared_types_pb2 as shared__types__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='quality_test.proto',
  package='prosuite.Microservices.Definitions.QA.Test',
  syntax='proto3',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x12quality_test.proto\x12*prosuite.Microservices.Definitions.QA.Test\x1a\x12shared_types.proto\"\xa5\x02\n\x12\x45xecuteTestRequest\x12K\n\nworkspaces\x18\x01 \x03(\x0b\x32\x37.prosuite.Microservices.Definitions.Shared.WorkspaceMsg\x12S\n\x0finvolved_tables\x18\x02 \x03(\x0b\x32:.prosuite.Microservices.Definitions.QA.Test.TestDatasetMsg\x12\x46\n\tperimeter\x18\x03 \x01(\x0b\x32\x33.prosuite.Microservices.Definitions.Shared.ShapeMsg\x12\x11\n\ttest_name\x18\x04 \x01(\t\x12\x12\n\nparameters\x18\x05 \x03(\t\"\xcf\x01\n\x13\x45xecuteTestResponse\x12\x1b\n\x13service_call_status\x18\x01 \x01(\x05\x12M\n\x08progress\x18\x02 \x01(\x0b\x32;.prosuite.Microservices.Definitions.QA.Test.TestProgressMsg\x12L\n\x06issues\x18\x03 \x03(\x0b\x32<.prosuite.Microservices.Definitions.QA.Test.DetectedIssueMsg\"\x80\x01\n\x0eTestDatasetMsg\x12S\n\x10\x63lass_definition\x18\x01 \x01(\x0b\x32\x39.prosuite.Microservices.Definitions.Shared.ObjectClassMsg\x12\x19\n\x11\x66ilter_expression\x18\x02 \x01(\t\"_\n\x0fTestProgressMsg\x12\x1c\n\x14progress_total_steps\x18\x01 \x01(\x05\x12\x1d\n\x15progress_current_step\x18\x02 \x01(\x05\x12\x0f\n\x07message\x18\x03 \x01(\t\"\xdb\x02\n\x10\x44\x65tectedIssueMsg\x12\x13\n\x0b\x64\x65scription\x18\x01 \x01(\t\x12K\n\x0eissue_geometry\x18\x02 \x01(\x0b\x32\x33.prosuite.Microservices.Definitions.Shared.ShapeMsg\x12\x15\n\rissue_code_id\x18\x03 \x01(\t\x12\x1e\n\x16issue_code_description\x18\x04 \x01(\t\x12X\n\x10involved_objects\x18\x05 \x03(\x0b\x32>.prosuite.Microservices.Definitions.QA.Test.InvolvedObjectsMsg\x12\x16\n\x0estop_condition\x18\x06 \x01(\x08\x12\x1a\n\x12\x61\x66\x66\x65\x63ted_component\x18\x07 \x01(\t\x12 \n\x18\x63reation_date_time_ticks\x18\x08 \x01(\x03\"t\n\x12InvolvedObjectsMsg\x12J\n\x07\x64\x61taset\x18\x01 \x01(\x0b\x32\x39.prosuite.Microservices.Definitions.Shared.ObjectClassMsg\x12\x12\n\nobject_ids\x18\x02 \x03(\x05\x32\xa2\x01\n\x0fQualityTestGrpc\x12\x8e\x01\n\x07\x45xecute\x12>.prosuite.Microservices.Definitions.QA.Test.ExecuteTestRequest\x1a?.prosuite.Microservices.Definitions.QA.Test.ExecuteTestResponse\"\x00\x30\x01\x62\x06proto3'
  ,
  dependencies=[shared__types__pb2.DESCRIPTOR,])




_EXECUTETESTREQUEST = _descriptor.Descriptor(
  name='ExecuteTestRequest',
  full_name='prosuite.Microservices.Definitions.QA.Test.ExecuteTestRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='workspaces', full_name='prosuite.Microservices.Definitions.QA.Test.ExecuteTestRequest.workspaces', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='involved_tables', full_name='prosuite.Microservices.Definitions.QA.Test.ExecuteTestRequest.involved_tables', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='perimeter', full_name='prosuite.Microservices.Definitions.QA.Test.ExecuteTestRequest.perimeter', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='test_name', full_name='prosuite.Microservices.Definitions.QA.Test.ExecuteTestRequest.test_name', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='parameters', full_name='prosuite.Microservices.Definitions.QA.Test.ExecuteTestRequest.parameters', index=4,
      number=5, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=87,
  serialized_end=380,
)


_EXECUTETESTRESPONSE = _descriptor.Descriptor(
  name='ExecuteTestResponse',
  full_name='prosuite.Microservices.Definitions.QA.Test.ExecuteTestResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='service_call_status', full_name='prosuite.Microservices.Definitions.QA.Test.ExecuteTestResponse.service_call_status', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='progress', full_name='prosuite.Microservices.Definitions.QA.Test.ExecuteTestResponse.progress', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='issues', full_name='prosuite.Microservices.Definitions.QA.Test.ExecuteTestResponse.issues', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=383,
  serialized_end=590,
)


_TESTDATASETMSG = _descriptor.Descriptor(
  name='TestDatasetMsg',
  full_name='prosuite.Microservices.Definitions.QA.Test.TestDatasetMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='class_definition', full_name='prosuite.Microservices.Definitions.QA.Test.TestDatasetMsg.class_definition', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='filter_expression', full_name='prosuite.Microservices.Definitions.QA.Test.TestDatasetMsg.filter_expression', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=593,
  serialized_end=721,
)


_TESTPROGRESSMSG = _descriptor.Descriptor(
  name='TestProgressMsg',
  full_name='prosuite.Microservices.Definitions.QA.Test.TestProgressMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='progress_total_steps', full_name='prosuite.Microservices.Definitions.QA.Test.TestProgressMsg.progress_total_steps', index=0,
      number=1, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='progress_current_step', full_name='prosuite.Microservices.Definitions.QA.Test.TestProgressMsg.progress_current_step', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='message', full_name='prosuite.Microservices.Definitions.QA.Test.TestProgressMsg.message', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=723,
  serialized_end=818,
)


_DETECTEDISSUEMSG = _descriptor.Descriptor(
  name='DetectedIssueMsg',
  full_name='prosuite.Microservices.Definitions.QA.Test.DetectedIssueMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='description', full_name='prosuite.Microservices.Definitions.QA.Test.DetectedIssueMsg.description', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='issue_geometry', full_name='prosuite.Microservices.Definitions.QA.Test.DetectedIssueMsg.issue_geometry', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='issue_code_id', full_name='prosuite.Microservices.Definitions.QA.Test.DetectedIssueMsg.issue_code_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='issue_code_description', full_name='prosuite.Microservices.Definitions.QA.Test.DetectedIssueMsg.issue_code_description', index=3,
      number=4, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='involved_objects', full_name='prosuite.Microservices.Definitions.QA.Test.DetectedIssueMsg.involved_objects', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='stop_condition', full_name='prosuite.Microservices.Definitions.QA.Test.DetectedIssueMsg.stop_condition', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='affected_component', full_name='prosuite.Microservices.Definitions.QA.Test.DetectedIssueMsg.affected_component', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='creation_date_time_ticks', full_name='prosuite.Microservices.Definitions.QA.Test.DetectedIssueMsg.creation_date_time_ticks', index=7,
      number=8, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=821,
  serialized_end=1168,
)


_INVOLVEDOBJECTSMSG = _descriptor.Descriptor(
  name='InvolvedObjectsMsg',
  full_name='prosuite.Microservices.Definitions.QA.Test.InvolvedObjectsMsg',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='dataset', full_name='prosuite.Microservices.Definitions.QA.Test.InvolvedObjectsMsg.dataset', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='object_ids', full_name='prosuite.Microservices.Definitions.QA.Test.InvolvedObjectsMsg.object_ids', index=1,
      number=2, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1170,
  serialized_end=1286,
)

_EXECUTETESTREQUEST.fields_by_name['workspaces'].message_type = shared__types__pb2._WORKSPACEMSG
_EXECUTETESTREQUEST.fields_by_name['involved_tables'].message_type = _TESTDATASETMSG
_EXECUTETESTREQUEST.fields_by_name['perimeter'].message_type = shared__types__pb2._SHAPEMSG
_EXECUTETESTRESPONSE.fields_by_name['progress'].message_type = _TESTPROGRESSMSG
_EXECUTETESTRESPONSE.fields_by_name['issues'].message_type = _DETECTEDISSUEMSG
_TESTDATASETMSG.fields_by_name['class_definition'].message_type = shared__types__pb2._OBJECTCLASSMSG
_DETECTEDISSUEMSG.fields_by_name['issue_geometry'].message_type = shared__types__pb2._SHAPEMSG
_DETECTEDISSUEMSG.fields_by_name['involved_objects'].message_type = _INVOLVEDOBJECTSMSG
_INVOLVEDOBJECTSMSG.fields_by_name['dataset'].message_type = shared__types__pb2._OBJECTCLASSMSG
DESCRIPTOR.message_types_by_name['ExecuteTestRequest'] = _EXECUTETESTREQUEST
DESCRIPTOR.message_types_by_name['ExecuteTestResponse'] = _EXECUTETESTRESPONSE
DESCRIPTOR.message_types_by_name['TestDatasetMsg'] = _TESTDATASETMSG
DESCRIPTOR.message_types_by_name['TestProgressMsg'] = _TESTPROGRESSMSG
DESCRIPTOR.message_types_by_name['DetectedIssueMsg'] = _DETECTEDISSUEMSG
DESCRIPTOR.message_types_by_name['InvolvedObjectsMsg'] = _INVOLVEDOBJECTSMSG
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

ExecuteTestRequest = _reflection.GeneratedProtocolMessageType('ExecuteTestRequest', (_message.Message,), {
  'DESCRIPTOR' : _EXECUTETESTREQUEST,
  '__module__' : 'quality_test_pb2'
  # @@protoc_insertion_point(class_scope:prosuite.Microservices.Definitions.QA.Test.ExecuteTestRequest)
  })
_sym_db.RegisterMessage(ExecuteTestRequest)

ExecuteTestResponse = _reflection.GeneratedProtocolMessageType('ExecuteTestResponse', (_message.Message,), {
  'DESCRIPTOR' : _EXECUTETESTRESPONSE,
  '__module__' : 'quality_test_pb2'
  # @@protoc_insertion_point(class_scope:prosuite.Microservices.Definitions.QA.Test.ExecuteTestResponse)
  })
_sym_db.RegisterMessage(ExecuteTestResponse)

TestDatasetMsg = _reflection.GeneratedProtocolMessageType('TestDatasetMsg', (_message.Message,), {
  'DESCRIPTOR' : _TESTDATASETMSG,
  '__module__' : 'quality_test_pb2'
  # @@protoc_insertion_point(class_scope:prosuite.Microservices.Definitions.QA.Test.TestDatasetMsg)
  })
_sym_db.RegisterMessage(TestDatasetMsg)

TestProgressMsg = _reflection.GeneratedProtocolMessageType('TestProgressMsg', (_message.Message,), {
  'DESCRIPTOR' : _TESTPROGRESSMSG,
  '__module__' : 'quality_test_pb2'
  # @@protoc_insertion_point(class_scope:prosuite.Microservices.Definitions.QA.Test.TestProgressMsg)
  })
_sym_db.RegisterMessage(TestProgressMsg)

DetectedIssueMsg = _reflection.GeneratedProtocolMessageType('DetectedIssueMsg', (_message.Message,), {
  'DESCRIPTOR' : _DETECTEDISSUEMSG,
  '__module__' : 'quality_test_pb2'
  # @@protoc_insertion_point(class_scope:prosuite.Microservices.Definitions.QA.Test.DetectedIssueMsg)
  })
_sym_db.RegisterMessage(DetectedIssueMsg)

InvolvedObjectsMsg = _reflection.GeneratedProtocolMessageType('InvolvedObjectsMsg', (_message.Message,), {
  'DESCRIPTOR' : _INVOLVEDOBJECTSMSG,
  '__module__' : 'quality_test_pb2'
  # @@protoc_insertion_point(class_scope:prosuite.Microservices.Definitions.QA.Test.InvolvedObjectsMsg)
  })
_sym_db.RegisterMessage(InvolvedObjectsMsg)



_QUALITYTESTGRPC = _descriptor.ServiceDescriptor(
  name='QualityTestGrpc',
  full_name='prosuite.Microservices.Definitions.QA.Test.QualityTestGrpc',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=1289,
  serialized_end=1451,
  methods=[
  _descriptor.MethodDescriptor(
    name='Execute',
    full_name='prosuite.Microservices.Definitions.QA.Test.QualityTestGrpc.Execute',
    index=0,
    containing_service=None,
    input_type=_EXECUTETESTREQUEST,
    output_type=_EXECUTETESTRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_QUALITYTESTGRPC)

DESCRIPTOR.services_by_name['QualityTestGrpc'] = _QUALITYTESTGRPC

# @@protoc_insertion_point(module_scope)



import os
from typing import Union, Iterable
from prosuite.condition import Condition
from prosuite.perimeter import EnvelopePerimeter, EsriShapePerimeter, WkbPerimeter
from prosuite.parameter import Parameter
from prosuite.transformed_dataset import TransformedDataset
from prosuite.xml_specification import XmlSpecification
from prosuite.specification import Specification
from prosuite.verification_parameters import VerificationParameters
from prosuite.advanced_parameters import AdvancedParameters
import prosuite.generated.quality_verification_service_pb2 as service_util
import prosuite.generated.quality_verification_service_pb2_grpc as qa_service
import prosuite.generated.shared_qa_pb2 as shared_qa
import prosuite.generated.shared_types_pb2 as shared_types
from prosuite.verification_response import VerificationResponse
import grpc
import logging
import sys

from prosuite.dataset import Dataset

MAX_MESSAGE_LENGTH_MB = 1024

class Service:
    """
    The service class communicates on the grpc channel and executes server functions

    :param host_name: host running the grpc quality verification service
    :type host_name: str
    :param port_nr: port used for the grpc quality verification service
    :type port_nr: int
    :param channel_credentials: ssl credentials add authentication between client and server and are handled on the channel level
    :type channel_credentials: str   
    """

    ISSUE_GDB = "Issues.gdb"
    XML_REPORT = "verification.xml"
    HTML_REPORT = "verification.html"

    def __init__(self, host_name: str, port_nr: int, channel_credentials: grpc.ssl_channel_credentials = None):
        #:
        self.host_name = host_name
        #:
        self.port_nr = port_nr
        #:
        self.ssl_channel_credentials: grpc.ssl_channel_credentials = channel_credentials

    def verify(self, specification: Union[Specification, XmlSpecification], perimeter: Union[EnvelopePerimeter,
                                                                                             EsriShapePerimeter, WkbPerimeter] = None, output_dir: str = None, parameters: VerificationParameters = None) -> Iterable[VerificationResponse]:
        """
        executes a verification. Runs all tests that are defined in the specification.
        Returns a collection of VerificationResponse objects, containing the verification
        messages.

        :return: Iterator for looping over VerificationResponse objects
        :rtype: Iterator[VerificationResponse]
        """

        advanced_parameters = AdvancedParameters(
            specification, output_dir, perimeter, parameters)

        self._validate_params(advanced_parameters)

        channel = self._create_channel()

        client = qa_service.QualityVerificationGrpcStub(channel)

        request = self._create_request(advanced_parameters)

        for response_msg in client.VerifyStandaloneXml(request):
            yield VerificationResponse(
                message=response_msg.message.message,
                service_call_status=response_msg.service_call_status,
                message_level=response_msg.message.message_level
            )

    def _create_request(self, parameters: AdvancedParameters):
        if isinstance(parameters.specification, XmlSpecification):
            request = self._compile_xml_request(parameters)
        else:
            request = self._compile_request(parameters)
        return request

    def _create_channel(self):
        message_length = MAX_MESSAGE_LENGTH_MB * 1024 * 1024
        options=[('grpc.max_send_message_length', message_length),
                 ('grpc.max_receive_message_length', message_length)]
        
        if self.ssl_channel_credentials:
            channel = self._create_secure_channel(options)
        else:
            channel = grpc.insecure_channel(f'{self.host_name}:{self.port_nr}', options)
        return channel

    def _validate_params(self, params: AdvancedParameters):
        if params.output_dir is None:
            params.output_dir = ""            
            logging.warn("No output dir is defined")
        if params.specification is None:
            raise Exception(
                "No specification is defined. Please assign verification.specification.")

    def _compile_xml_request(self, parameters: AdvancedParameters):
        if not parameters.specification.specification_name:
            raise Exception(
                "The specification to be used is not defined. Please define XmlSpecification.specification_name")
        req = service_util.StandaloneVerificationRequest()
        req.output_directory = parameters.output_dir
        self._configure_verification_parameter_msg(req.parameters, parameters)
        self._configure_xml_quality_specification_msg(
            req.xml_specification, parameters.specification)
        return req

    def _compile_request(self, parameters: AdvancedParameters):
        req = service_util.StandaloneVerificationRequest()
        req.output_directory = parameters.output_dir
        self._configure_verification_parameter_msg(req.parameters, parameters)
        self._configure_condition_list_specification_msg(
            req.condition_list_specification, parameters.specification)
        return req

    def _create_secure_channel(self, options) -> grpc.Channel:
        channel = grpc.secure_channel(
            f'{self.host_name}:{self.port_nr}', self.ssl_channel_credentials, options)
        try:
            grpc.channel_ready_future(channel).result(timeout=5)
            logging.info(
                f'Successfully established secure channel to {self.host_name}')
        except:
            logging.exception(
                f'Timeout. Failed to establish secure channel to {self.host_name}')
        return channel

    def _configure_xml_quality_specification_msg(self, xml_specification_msg: shared_qa.XmlQualitySpecificationMsg, specification: XmlSpecification):
        xml_specification_msg.xml = specification.xml_string
        xml_specification_msg.selected_specification_name = specification.specification_name
        if specification.data_source_replacements:
            xml_specification_msg.data_source_replacements.extend(
                specification.data_source_replacements)
        return xml_specification_msg

    def _configure_condition_list_specification_msg(self,
                                                    cond_list_spec_msg: shared_qa.ConditionListSpecificationMsg, specification: Specification):
        cond_list_spec_msg.name = specification.name
        cond_list_spec_msg.description = specification.description
        for condition in specification.get_conditions():
            cond_list_spec_msg.elements.append(
                self._to_xml_quality_specification_element_msg(condition))

        data_sources = self._get_data_sources(specification)

        for key, value in data_sources.items():
            data_source_msg = shared_qa.DataSourceMsg()
            data_source_msg.id = key
            data_source_msg.model_name = key
            data_source_msg.catalog_path = value
            cond_list_spec_msg.data_sources.append(data_source_msg)

        return cond_list_spec_msg

    def _configure_shape_msg(self, shape_msg: shared_types.ShapeMsg, perimeter: Union[EnvelopePerimeter, WkbPerimeter, EsriShapePerimeter]):
        if isinstance(perimeter, EnvelopePerimeter):
            perimeter: EnvelopePerimeter
            shape_msg.envelope.x_min = perimeter.x_min
            shape_msg.envelope.x_max = perimeter.x_max
            shape_msg.envelope.y_min = perimeter.y_min
            shape_msg.envelope.y_max = perimeter.y_max
        if isinstance(perimeter, EsriShapePerimeter):
            perimeter: EsriShapePerimeter
            shape_msg.esri_shape = perimeter.esri_shape
        if isinstance(perimeter, WkbPerimeter):
            perimeter: WkbPerimeter
            shape_msg.wkb = bytes(perimeter.wkb)

    def _to_xml_quality_specification_element_msg(self, condition: Condition):
        spec_element = shared_qa.QualitySpecificationElementMsg()
        self._configure_quality_condition_msg(
            condition, spec_element.condition)
        spec_element.allow_errors = condition.allow_errors
        spec_element.category_name = condition.category
        # other props
        return spec_element

    def _configure_verification_parameter_msg(self, params_msg: shared_qa.VerificationParametersMsg, parameters: AdvancedParameters):
        params_msg.tile_size = parameters.tile_size
        params_msg.issue_file_gdb_path = os.path.join(
            parameters.output_dir, Service.ISSUE_GDB)
        params_msg.html_report_path = os.path.join(
            parameters.output_dir, Service.HTML_REPORT)
        params_msg.verification_report_path = os.path.join(
            parameters.output_dir, Service.XML_REPORT)
        if parameters.perimeter:
            self._configure_shape_msg(
                params_msg.perimeter, parameters.perimeter)

    def _configure_quality_condition_msg(self, condition: Condition, condition_msg: shared_qa.QualityConditionMsg):
        condition_msg.name = condition.name
        condition_msg.test_descriptor_name = condition.test_descriptor
        condition_msg.description = condition.description
        condition_msg.url = condition.url
        for param in condition.parameters:
            if param.contains_list_of_datasets:
                self._handle_dataset_list(condition_msg, param)
            else:
                condition_msg.parameters.append(self._to_parameter_mgs(param))

    def _handle_dataset_list(self, condition_msg: shared_qa.QualityConditionMsg, param: Parameter):
        """
        a Parameter value can be of type Dataset. Or it can be of type list[Dataset].
        if it is a Dataset list, each Dataset in the list should be treated as single Parameter
        """
        if param.contains_list_of_datasets:
            # in this case, param.dataset is actually a list of datasets. we need to unpack the list and create
            # single params for each dataset in the list
            if Parameter.value_is_list_of_datasets(param.dataset):
                for parameter in param.dataset:
                    ds_param = Parameter(param.name, parameter)
                    condition_msg.parameters.append(
                        self._to_parameter_mgs(ds_param))

    @staticmethod
    def _to_parameter_mgs(param: Parameter):
        param_msg = shared_qa.ParameterMsg()
        param_msg.name = param.name

        if isinstance(param.value, TransformedDataset):
            # handle transformed dataset here
            param_msg.where_clause = Service._none_to_emtpy_str(
                param.get_where_clause())
        else:
            param_msg.value = Service._none_to_emtpy_str(
                param.get_string_value())
            param_msg.where_clause = Service._none_to_emtpy_str(
                param.get_where_clause())
            param_msg.workspace_id = param.get_workspace_id()
            param_msg.used_as_reference_data = False

        return param_msg

    @staticmethod
    def _none_to_emtpy_str(value) -> str:
        if value is None:
            return ""
        return value

    def _get_data_sources(self, specification: Specification) -> dict:
        result = {}
        for condition in specification.get_conditions():
            for parameter in condition.parameters:
                if parameter.is_dataset_parameter():
                    if parameter.contains_list_of_datasets:
                        for dataset in parameter.dataset:
                            model = dataset.model
                            result[model.name] = model.catalog_path
                        pass
                    else:
                        model = parameter.dataset.model
                        result[model.name] = model.catalog_path
        return result



import setuptools
import sys

# build the prosuite package.
# version parameter is read from setup.cfg

setuptools.setup(
    name="prosuite",
    author="Dira GeoSystems",
    author_email="programmers@dirageosystems.ch",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
        "Development Status :: 5 - Production/Stable"
    ],
    url="https://dirageosystems.ch/prosuite.html",
    packages=['prosuite', 'prosuite.generated',
              'prosuite.datatypes', 'prosuite.factories'],
    package_data={'': ['*.txt']},
    install_requires=["lxml", "grpcio>1.4.0", "grpcio-tools"],
    description='An API to configure quality assurance tests for geodata and execute quality verifications on a ProSuite Server',
    long_description="""ProSuite API for Python
     ProSuite is a suite of high-end productivity tools for ArcGIS for data production,
     quality assurance, and cartographic refinement.The ProSuite API for Python allows to configure quality assurance tests for geodata and 
     execute quality verifications on a ProSuite Server. For further information please contact: hello@dirageosystems.ch
    """,
    keywords="gis, arcgis, geodata, prosuite, quality, quality-assurance, qa, test"
)



from prosuite.model import Model


class BaseDataset():
    def __init__(self, name: str, filter_expression: str = ""):
        self.name: str = name
        self.filter_expression: str = filter_expression
        pass


class Dataset(BaseDataset):
    """
    A dataset represents a table in an esri workspace. Usually the table is a featureclass.
    """

    def __init__(self, name: str, model: Model, filter_expression: str = ""):
        """
        :param name: table (featureclass) name
        :param model: prosuite.model
        :param filter_expression: a where clause that filters the table. syntax of the whereclause is defined in
        <prosuite installation folder>/doc/tests/SQLSyntax_en.pdf
        """
        super().__init__(name, filter_expression)
        self.model: Model = model



# generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

import quality_test_pb2 as quality__test__pb2


class QualityTestGrpcStub(object):
    """*
    The quality test definition.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Execute = channel.unary_stream(
                '/prosuite.Microservices.Definitions.QA.Test.QualityTestGrpc/Execute',
                request_serializer=quality__test__pb2.ExecuteTestRequest.SerializeToString,
                response_deserializer=quality__test__pb2.ExecuteTestResponse.FromString,
                )


class QualityTestGrpcServicer(object):
    """*
    The quality test definition.
    """

    def Execute(self, request, context):
        """*
        The quality verification RPC.

        *
        Verifies the quality definied in the data dictionary.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_QualityTestGrpcServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Execute': grpc.unary_stream_rpc_method_handler(
                    servicer.Execute,
                    request_deserializer=quality__test__pb2.ExecuteTestRequest.FromString,
                    response_serializer=quality__test__pb2.ExecuteTestResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'prosuite.Microservices.Definitions.QA.Test.QualityTestGrpc', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class QualityTestGrpc(object):
    """*
    The quality test definition.
    """

    @staticmethod
    def Execute(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/prosuite.Microservices.Definitions.QA.Test.QualityTestGrpc/Execute',
            quality__test__pb2.ExecuteTestRequest.SerializeToString,
            quality__test__pb2.ExecuteTestResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)
